/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>
 * Ryan Barrett <p4@ryanb.org>
 * Maulik Shah <p4@maulik.net>
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file sys.cpp
 *
 * Contain functions for cross-platform portability.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "sys.h"

#ifdef _WIN32
#include <errno.h>
#include <pthread.h>

/**
 * gettimeofday() -- gets the current time in elapsed seconds and
 * microsends since GMT Jan 1, 1970.
 *
 * ARGUMENTS: - Pointer to a timeval struct to return the time into
 *
 * RETURN CODES: - 0 on success
 * -1 on failure
 *
 * We got this code from:
 * http://www.freecode.com/viewsource/webstone/WebStone2.5-src::src::gettimeofday.c/
 */
int gettimeofday(struct timespec *curTimeP, int *junk)
{
  struct _timeb localTime;

  if (curTimeP == NULL) {
    errno = EFAULT;
    return (-1);
  }

 /**
  * Compute the elapsed time since Jan 1, 1970 by first
  * obtaining the elapsed time from the system using the
  * _ftime(..) call and then convert to the "timeval"
  * equivalent.
  */

  _ftime(&localTime);

  curTimeP->tv_sec = localTime.time + localTime.timezone;
  curTimeP->tv_nsec = localTime.millitm * 1000;

  return(0);
}

#endif // _WIN32
